--	LÖVE Code Doodle #13
--		Lights Off (or something like this)

function love.load()
	windowsW, windowsH = 800, 600
	love.graphics.setMode = love.graphics.setMode or love.window.setMode
	love.graphics.setMode(windowsW, windowsH)
	
	love.graphics.setCaption = love.graphics.setCaption or love.window.setTitle
	love.graphics.setCaption("Lights Off - by HugoBDesigner")
	
	map = {}
	
	mapsizes = {5, 20} --min and max
	mapsize = 10
	
	randomstepsv = {10, 50} --min and max
	randomsteps = 20
	
	font1 = love.graphics.newFont(12)
	font2 = love.graphics.newFont(36)
	
	buttontimer = {timer = 0, maxtimer = .5, repeattimer = 0, repeatmaxtimer = .05}
	
	buttons = {}
	
	local t1 = 5+font1:getWidth("Map size: ")
	table.insert(buttons, {x = t1, y = windowsH-20-5, width = 20, height = 20, text = "<", func = decreaseMap})
	
	local t2 = t1+20+font1:getWidth(" " .. tostring(mapsize) .. " ")
	table.insert(buttons, {x = t2, y = windowsH-20-5, width = 20, height = 20, text = ">", func = increaseMap})
	
	local t3 = t2+20+30+font1:getWidth("Randomization steps: ")
	table.insert(buttons, {x = t3, y = windowsH-20-5, width = 20, height = 20, text = "<", func = decreaseSteps})
	
	local t4 = t3+20+font1:getWidth(" " .. tostring(randomsteps) .. " ")
	table.insert(buttons, {x = t4, y = windowsH-20-5, width = 20, height = 20, text = ">", func = increaseSteps})
	
	winMessage = false
	
	recsizes = {5, 30} --min and max
	recsize = 15
	recx = 0
	recy = 0
	color1 = {0, 35, 55, 255}
	color2 = {155, 255, 55, 255}
	
	newMap()
	
	love.graphics.setBackgroundColor(255, 255, 255, 255)
end

function love.update(dt)
	local x, y = love.mouse.getPosition()
	if love.mouse.isDown("l") then
		local ins = false
		for i, v in ipairs(buttons) do
			if inside(x, y, 0, 0, v.x, v.y, v.width, v.height) then
				ins = true
				break
			end
		end
		
		if ins then
			if buttontimer.timer < buttontimer.maxtimer then
				buttontimer.timer = buttontimer.timer+dt
			else
				buttontimer.repeattimer = buttontimer.repeattimer+dt
				
				if buttontimer.repeattimer >= buttontimer.repeatmaxtimer then
					buttontimer.repeattimer = 0
					love.mousepressed(x, y, "l")
				end
			end
		else
			buttontimer.timer = 0
			buttontimer.repeattimer = 0
		end
	else
		buttontimer.timer = 0
		buttontimer.repeattimer = 0
	end
end

function love.draw()
	love.graphics.setLineWidth(1)
	for x = 1, #map do
		for y = 1, #map[x] do
			local px = (x-1)*recsize+recx
			local py = (y-1)*recsize+recy
			
			if map[x][y] then
				love.graphics.setColor(color2)
			else
				love.graphics.setColor(color1)
			end
			
			love.graphics.rectangle("fill", px, py, recsize, recsize)
			
			love.graphics.setColor(0, 0, 0, 55)
			love.graphics.rectangle("line", px, py, recsize, recsize)
		end
	end
	love.graphics.setLineWidth(3)
	love.graphics.setColor(0, 0, 0, 255)
	love.graphics.rectangle("line", recx, recy, #map*recsize, #map[1]*recsize)
	
	love.graphics.setLineWidth(1)
	
	love.graphics.setFont(font1)
	love.graphics.setColor(0, 0, 0, 255)
	local yy = buttons[1].y+buttons[1].height/2-font1:getHeight()/2
	
	love.graphics.print("Map size: ", 5, yy)
	love.graphics.print(" " .. tostring(mapsize) .. " ", buttons[1].x+buttons[1].width, yy)
	love.graphics.print("Randomization steps: ", buttons[2].x+buttons[2].width+30, yy)
	love.graphics.print(" " .. tostring(randomsteps) .. " ", buttons[3].x+buttons[3].width, yy)
	love.graphics.print(" Ctrl+N to start a new game", buttons[4].x+buttons[4].width+30, yy)
	
	for i, v in ipairs(buttons) do
		love.graphics.setColor(155, 175, 255, 255)
		love.graphics.rectangle("fill", v.x, v.y, v.width, v.height)
		
		love.graphics.setColor(0, 0, 0, 255)
		love.graphics.rectangle("line", v.x, v.y, v.width, v.height)
		love.graphics.print(v.text, v.x+v.width/2-font1:getWidth(v.text)/2, v.y+v.height/2-font1:getHeight()/2)
	end
	
	love.graphics.setLineWidth(3)
	love.graphics.setColor(55, 55, 55, 255)
	local ww = 70
	local yy = windowsH-21
	local wa, ha = 5, 11
	
	love.graphics.print("Zoom: ", windowsW-5-ww-font1:getWidth("Zoom: "), yy-font1:getHeight()/2)
	
	love.graphics.setColor(0, 0, 0, 255)
	local text = "(Mouse wheel; Ctrl to zoom faster)"
	love.graphics.print(text, windowsW-5-font1:getWidth(text), yy+font1:getHeight()-8)
	
	love.graphics.line(windowsW-5-ww, yy, windowsW-5, yy)
	
	local x = (recsize-recsizes[1])/(recsizes[2]-recsizes[1])
	love.graphics.setColor(155, 155, 155, 155)
	love.graphics.rectangle("fill", windowsW-5-ww-wa/2+x*ww, yy-ha/2, wa, ha)
	love.graphics.setLineWidth(1)
	love.graphics.setColor(0, 0, 0, 255)
	love.graphics.rectangle("line", windowsW-5-ww-wa/2+x*ww, yy-ha/2, wa, ha)
	
	if winMessage then
		love.graphics.setColor(0, 0, 0, 155)
		love.graphics.rectangle("fill", 0, 0, windowsW, windowsH-5-20-5)
		love.graphics.setFont(font2)
		
		for a = 1, 2 do
		local text = "Congratulations! You win!"
		love.graphics.print(text, windowsW/2-font2:getWidth(text)/2, windowsH/2-font2:getHeight()-5)
		
		text = "Press ENTER to start a new game"
		love.graphics.print(text, windowsW/2-font2:getWidth(text)/2, windowsH/2+font2:getHeight()+5)
		love.graphics.setColor(205, 255, 0, 255)
		end
	end
end

function inside(x1, y1, w1, h1, x2, y2, w2, h2)
	if x1 >= x2 and y1 >= y2 and x1+w1 <= x2+w2 and y1+h1 <= y2+h2 then
		return true
	end
	return false
end

function love.mousepressed(x, y, button)
	local nx, ny = x-recx, y-recy
	nx = math.floor(nx/recsize)+1
	ny = math.floor(ny/recsize)+1
	
	local ctrlpressed = love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")
	if button == "l" then
		if nx >= 1 and nx <= #map and ny >= 1 and ny <= #map[1] and not winMessage then
			clickMap(nx, ny)
		end
		
		for i, v in ipairs(buttons) do
			if inside(x, y, 0, 0, v.x, v.y, v.width, v.height) then
				v.func()
			end
		end
	elseif button == "wu" then
		local n = 1
		if ctrlpressed then n = 5 end
		recsize = math.min(recsizes[2], recsize+n)
		updateRecs()
	elseif button == "wd" then
		local n = 1
		if ctrlpressed then n = 5 end
		recsize = math.max(recsizes[1], recsize-n)
		updateRecs()
	end
end

function love.keypressed(key, unicode)
	local ctrlpressed = love.keyboard.isDown("lctrl") or love.keyboard.isDown("rctrl")
	
	if ctrlpressed then
		if key == "n" then
			newMap()
			winMessage = false
		elseif key == "i" then
			for x = 1, #map do
				for y = 1, #map[x] do
					map[x][y] = not map[x][y]
				end
			end
		elseif key == "a" then
			for y = 1, #map[1] do
				for x = 1, #map do
					clickMap(x, y, true)
				end
			end
		end
	end
	
	if (key == "enter" or key == "return" or key == "kpenter") and winMessage then
		newMap()
		winMessage = false
	elseif key == "escape" then
		love.event.quit()
	end
end

function updateRecs()
	recx = windowsW/2-(recsize*#map)/2
	recy = windowsH/2-(recsize*#map[1])/2
end

function newMap()
	map = {}
	for x = 1, mapsize do
		map[x] = {}
		for y = 1, mapsize do
			map[x][y] = false
		end
	end
	
	updateRecs()
	
	randomizeMap()
end

function randomizeMap()
	for i = 1, randomsteps do
		clickMap(math.random(mapsize), math.random(mapsize), true)
	end
end

function clickMap(x, y, m)
	local m = m or false
	map[x][y] = not map[x][y]
	if x-1 >= 1 then
		map[x-1][y] = not map[x-1][y]
	end
	if x+1 <= #map then
		map[x+1][y] = not map[x+1][y]
	end
	if y-1 >= 1 then
		map[x][y-1] = not map[x][y-1]
	end
	if y+1 <= #map[1] then
		map[x][y+1] = not map[x][y+1]
	end
	
	if not m then
		checkMap()
	end
end

function checkMap()
	local win = true
	
	for x = 1, #map do
		for y = 1, #map[x] do
			if map[x][y] then
				win = false
				break
			end
		end
		if not win then
			break
		end
	end
	
	if win then
		winMessage = true
	end
end

function increaseMap()
	mapsize = math.min(mapsize+1, mapsizes[2])
end

function decreaseMap()
	mapsize = math.max(mapsize-1, mapsizes[1])
end

function increaseSteps()
	randomsteps = math.min(randomsteps+1, randomstepsv[2])
end

function decreaseSteps()
	randomsteps = math.max(randomsteps-1, randomstepsv[1])
end